import time

from . import common as common
from . import constant as constant
from . import manage_alb as alb_manager
from . import manage_cloudwatch as cloudwatch_manager
from . import manage_ecr as ecr_manager


###
#   Classe permettant de gérer le déployement
###
class DeploymentManager:
    alb = None
    http_listener = None
    default_target_group = None
    rules = []
    repositories = []
    prod_color = None
    blue_environment = {}
    green_environment = {}

    # Clients
    elbv2_client = None

    def __init__(self, elbv2_client, alb, http_listener, rules, repositories,
                 prod_color, current_target_group_type,
                 blue_environment,
                 green_environment):
        self.elbv2_client = elbv2_client
        self.alb = alb
        self.http_listener = http_listener
        self.rules = rules
        self.repositories = repositories
        self.prod_color = prod_color
        self.current_target_group_type = current_target_group_type
        self.blue_environment = blue_environment
        self.green_environment = green_environment

    # Constuit une action pour le listener
    def __build_forward_actions(self, target_group_arn):
        return {
            "Type": "forward",
            "TargetGroupArn": target_group_arn,
            "Order": 1
        }

    def get_type(self, rule):
        type_holder = self.http_listener['ListenerArn'] if rule['IsDefault'] else rule['RuleArn']
        target_type = alb_manager.get_type_from_resource(type_holder)

        if target_type:
            return target_type
        else:
            return None

    def get_production_environment(self):
        if self.prod_color == constant.BLUE:
            return self.blue_environment
        elif self.prod_color == constant.GREEN:
            return self.green_environment
        raise Exception('Unable to get prod environment...')

    def get_pre_production_environment(self):
        if self.prod_color == constant.GREEN:
            return self.blue_environment
        elif self.prod_color == constant.BLUE:
            return self.green_environment
        raise Exception('Unable to get pre prod environment...')

    def create_rule(self, conditions, actions, priority, tags):
        self.elbv2_client.create_rule(
            ListenerArn=self.http_listener['ListenerArn'],
            Conditions=conditions,
            Actions=actions,
            Priority=priority,
            Tags=tags
        )

    def update_rule_target_group(self, expected_rule_type, expected_rule_color, new_target_group_arn):
        targeted_rules = self.get_rules_with_type_and_color(expected_rule_type, expected_rule_color)
        for targeted_rule in targeted_rules:
            self.__modify_rule_target_group(targeted_rule, new_target_group_arn)

    def update_rules_target_group(self, rules, new_target_group_arn):
        for rule in rules:
            self.__modify_rule_target_group(rule, new_target_group_arn)

    def __modify_rule_target_group(self, rule, target_group_arn):
        if rule['IsDefault']:
            return self.elbv2_client.modify_listener(
                ListenerArn=self.http_listener['ListenerArn'],
                DefaultActions=[self.__build_forward_actions(target_group_arn)]
            )
        else:
            return self.elbv2_client.modify_rule(
                RuleArn=rule['RuleArn'],
                Actions=[self.__build_forward_actions(target_group_arn)]
            )

    def get_rules_with_type_and_color(self, expected_type, expected_color):
        return [r for r in self.rules if self.__assert_rule(r, expected_type, expected_color)]

    def get_forward_rules(self):
        return self.get_typed_rules('forward')

    def get_fixed_response_rules(self):
        return self.get_typed_rules('fixed-response')

    def get_typed_rules(self, rule_type):
        typed_rules = []
        for rule in self.rules:
            for action in rule['Actions']:
                if action['Type'] == rule_type:
                    typed_rules.append(rule)
        return typed_rules

    def __assert_rule(self, rule, expected_type, expected_color):
        expected = (expected_type, expected_color)

        for action in rule['Actions']:
            if action['Type'] == 'forward':
                tap_tpl = common.get_type_and_color_for_resource(action['TargetGroupArn'], self.elbv2_client)
                if tap_tpl == expected:
                    return True

        return False

    def __assert_target_group(self, target_group, expected_type, expected_color):

        return target_group['Type'].upper() == expected_type.upper() \
            and target_group['Color'].upper() == expected_color.upper()

    # Ajout d'un tag a tous les repository d'un environement
    def add_tag_to_repositories(self, tag):
        for r in self.repositories:
            r.add_tag(tag)

    def set_color_to_list_repositories_name(self, repositories_name):
        print('Add color {} to mismatched repositories: {}'.format(self.prod_color, repositories_name))

        for r in self.repositories:
            if r.name in repositories_name:
                r.add_tag(self.prod_color.upper())

    # Cherche les repositories qui ont un tag mais pour lesquels la couleur active n'est pas appliquée et applique la
    def find_mismatched_repositories_name_between_tag_and_active_color(self, tag):
        return ecr_manager.find_mismatched_repositories_between_tag_and_color(
            ecr_manager.get_service_repositories_name(),
            tag,
            self.prod_color)

    def get_lowest_available_priority_alb_rule(self):
        used_priorities = set()

        for rule in self.rules:
            if not rule['IsDefault']:
                used_priorities.add(int(rule['Priority']))

        # Trouver la plus petite priorité disponible
        for priority in range(1, 50001):  # Les priorités ALB vont de 1 à 50000
            if priority not in used_priorities:
                return priority

        # Si toutes les priorités sont utilisées (cas très improbable)
        return None


###
#   Classe contenant les informations utiles d'un environment blue ou green
###
class Environment:
    ecs_client = None
    workspace = None
    color = None
    target_group_type = None
    cluster_name = None
    ecs_services = []
    target_group_arn = None

    def __init__(self, ecs_client, workspace, color, target_group_type, cluster_name, ecs_services,
                 target_group_arn):
        self.ecs_client = ecs_client
        self.workspace = workspace
        self.color = color
        self.target_group_type = target_group_type
        self.cluster_name = cluster_name
        self.ecs_services = ecs_services
        self.target_group_arn = target_group_arn

    # Démarre tous les services
    def start_up_services(self, desired_count=None):
        for s in self.ecs_services:
            s.start(desired_count)
        # Wait for all service receive startup
        time.sleep(10)

    # Eteint tous les services
    def shutdown_services(self):
        for s in self.ecs_services:
            s.shutdown()
        # Wait for all service receive shutdown
        time.sleep(10)

    def get_unhealthy_services(self):
        return list(filter(lambda s: not s.is_service_healthy(), self.ecs_services))

    # Vérifie que tous les services sont healthy
    def all_services_are_healthy(self):
        return all(s.is_service_healthy() for s in self.ecs_services)

    # Attend que tous les services soit healthy
    def wait_for_services_health(self):
        retry = 1
        print("Waiting {} seconds before first try".format(constant.HEALTHCHECK_SLEEPING_TIME))
        time.sleep(constant.HEALTHCHECK_SLEEPING_TIME)
        while not self.all_services_are_healthy() and constant.HEALTHCHECK_RETRY_LIMIT >= retry:
            print("Retry number {} all services hasnt healthy sleeping {} seconds before retry"
                  .format(retry, constant.HEALTHCHECK_SLEEPING_TIME))
            retry = retry + 1
            time.sleep(constant.HEALTHCHECK_SLEEPING_TIME)
        if constant.HEALTHCHECK_RETRY_LIMIT < retry:
            print("Tried {} but retry limit has been reach before all services been healthy".format(retry))
            # Raise exception
            unhealthy_sve = ",".join(list(map(lambda a: a.service_arn, self.get_unhealthy_services())))
            raise Exception("Unable to deploy, services still unhealthy. Unhealthy Services : {}".format(unhealthy_sve))
        else:
            print("Tried {} and all service are now healthy".format(retry))

    def get_active_and_pending_smuggler_jobs(self):
        return cloudwatch_manager.get_smuggler_metrics(self.workspace, self.color)


###
# Classe qui map un ECS aws
###
class EcsService:
    cluster_name = None
    service_arn = None
    task = []
    ecs_client = None
    service_healthy = False
    application_autoscaling_client = None
    max_capacity = None
    resource_id = None

    def __init__(self, ecs_client, application_autoscaling_client, cluster_name, service_arn, max_capacity,
                 resource_id):
        self.ecs_client = ecs_client
        self.cluster_name = cluster_name
        self.service_arn = service_arn
        self.application_autoscaling_client = application_autoscaling_client
        self.max_capacity = max_capacity
        self.resource_id = resource_id

    def __get_task(self):
        tasks = self.ecs_client.list_tasks(
            cluster=self.cluster_name,
            serviceName=self.service_arn,
            # TODO: Review if one day we got more than 100 ecs tasks !
            maxResults=100
        )
        return tasks['taskArns']

    def __set_register_scalable_target(self, min_capacity):
        try:
            return self.application_autoscaling_client.register_scalable_target(
                ServiceNamespace=constant.ECS_SERVICE_NAMESPACE,
                ResourceId=self.resource_id,
                ScalableDimension=constant.DEFAULT_SCALABLE_DIMENSION,
                MinCapacity=min_capacity,
                MaxCapacity=self.max_capacity
            )
        except Exception as err:
            print("An exception was raise during creation of new scalable target. Error : {}".format(err))

    def start(self, desired_count=None):
        if not desired_count:
            desired_count = constant.DEFAULT_DESIRED_COUNT
        print('Start service {} with {} instances'.format(self.service_arn, desired_count))
        self.ecs_client.update_service(
            cluster=self.cluster_name,
            service=self.service_arn,
            desiredCount=desired_count,
            forceNewDeployment=True
        )
        response = self.__set_register_scalable_target(desired_count)
        print("Started service: '{}', Updated Capacities => MaxCapacity: {} / MinCapacity: {}, response: {}"
              .format(self.service_arn, self.max_capacity, desired_count, response))

    def shutdown(self):
        print('Shutdown service {}'.format(self.service_arn))

        response = self.__set_register_scalable_target(0)
        print("Disabled autoscaling for service: '{}', Updated Capacities => MaxCapacity: {} / MinCapacity: 0, response: {}"
              .format(self.service_arn, self.max_capacity, response))

        self.ecs_client.update_service(
            cluster=self.cluster_name,
            service=self.service_arn,
            desiredCount=0
        )
        print("Stopped service: '{}'".format(self.service_arn))

    def is_service_healthy(self):
        if not self.service_healthy:
            self.service_healthy = self.__check_service_health()
        return self.service_healthy

    def __check_service_health(self):
        tasks = self.__get_task()
        if not tasks:
            return False
        detailed_task = self.ecs_client.describe_tasks(
            cluster=self.cluster_name,
            tasks=tasks
        )
        nb_healthy_task = len(list(filter(lambda x: x['healthStatus'] == 'HEALTHY', detailed_task['tasks'])))
        is_healthy = nb_healthy_task >= constant.DEFAULT_DESIRED_COUNT
        if is_healthy:
            print('{} has reach the healthy state'.format(self.service_arn))
        else:
            print('{} is not healthy, only has {} task(s) healthy and {} healthy tasks are required to pass'
                  .format(self.service_arn, nb_healthy_task, constant.DEFAULT_DESIRED_COUNT))
        return is_healthy

    def __str__(self):
        return self.service_arn


###
# Classe qui map un ECR aws
###
class Repository:
    name = None
    image = None
    manifest = None
    ecr_client = None

    def __init__(self, ecr_client, name, image, manifest):
        self.ecr_client = ecr_client
        self.name = name
        self.image = image
        self.manifest = manifest

    def add_tag(self, tag):
        try:
            print('Adding tag {} to image {} in repository {}'.format(tag, self.image, self.name))
            new_image = self.ecr_client.put_image(
                repositoryName=self.name,
                imageManifest=self.manifest,
                imageTag=tag
            )
            return new_image
        except self.ecr_client.exceptions.ImageAlreadyExistsException:
            print('Image {} in repository {} already exist with tag {}'.format(self.image, self.name, tag))



from .deployment_manager \
    import DeploymentManager, Repository, Environment, EcsService
from . import manage_ecr as ecr_manager
from . import manage_alb as alb_manager
from . import manage_ecs as ecs_manager
from . import constant as constant
import boto3

# Client
ecr_client = boto3.client('ecr')
ecs_client = boto3.client('ecs')
elbv2_client = boto3.client('elbv2')
application_autoscaling_client = boto3.client('application-autoscaling')


def build_deployment_manager(alb_name, cluster_name, img_deploy_tag, ssl_enabled, workspace):
    alb = alb_manager.get_alb_from_aws(alb_name)
    listener = alb_manager.get_current_listener(alb['LoadBalancerArn'], ssl_enabled)
    rules = alb_manager.get_uncolored_rules(listener)
    prod_color = alb_manager.get_production_color(listener)
    current_target_group_type = alb_manager.get_production_type(listener)
    repositories = list(
        map(lambda x: __build_repository(x, img_deploy_tag), ecr_manager.get_service_repositories_name()))
    green_environment = __build_environment(constant.GREEN, current_target_group_type,
                                            cluster_name, workspace)
    blue_environment = __build_environment(constant.BLUE, current_target_group_type,
                                           cluster_name, workspace)

    return DeploymentManager(
        elbv2_client=elbv2_client,
        alb=alb,
        http_listener=listener,
        rules=[r for r in rules if r],
        prod_color=prod_color,
        current_target_group_type=current_target_group_type,
        repositories=[r for r in repositories if r],
        green_environment=green_environment,
        blue_environment=blue_environment,
    )


def __build_repository(repository_name, tag):
    image = ecr_manager.get_repository_image_for_tag(repository_name, tag)
    if image:
        image_manifest = ecr_manager.get_image_manifest(repository_name, image)
        return Repository(
            name=repository_name,
            ecr_client=ecr_client,
            image=image,
            manifest=image_manifest
        )


def build_service(cluster_name, service_arn):
    return EcsService(ecs_client=ecs_client, application_autoscaling_client=application_autoscaling_client,
                      cluster_name=cluster_name, service_arn=service_arn,
                      max_capacity=ecs_manager.get_service_max_capacity_from_service_arn(service_arn),
                      resource_id=ecs_manager.get_service_resource_id_from_service_arn(service_arn))


def __build_environment(color, target_group_type, cluster_name, workspace):
    services_arn = ecs_manager.get_services_arn_for_color(color, cluster_name)
    ecs_services = list(map(
        lambda x: build_service(cluster_name, x),
        services_arn
    ))

    return Environment(
        workspace=workspace,
        color=color,
        target_group_type=target_group_type,
        cluster_name=cluster_name,
        ecs_client=ecs_client,
        ecs_services=[s for s in ecs_services if s],
        target_group_arn=alb_manager.get_target_group_with_type_color_and_workspace(
            target_group_type, color, workspace
        )
    )



import boto3
from . import constant as constant

# Client
elbv2_client = boto3.client('elbv2')
tagging_client = boto3.client('resourcegroupstaggingapi')


# ~~~~~~~~~~~~~~~~ ALB ~~~~~~~~~~~~~~~~

def get_alb_from_aws(alb_name):
    """
    Récupère l'application load balancer sur aws
    :param alb_name:    Nom du load balance
    :type alb_name:     str
    :return:            Load balancer trouvé
    :rtype:             dict
    """
    alb_desc = elbv2_client.describe_load_balancers(
        Names=[alb_name]
    )
    # WARNING: If we got multiple alb
    return alb_desc['LoadBalancers'][0]


# ~~~~~~~~~~~~~~~~ Listener ~~~~~~~~~~~~~~~~

def get_current_listener(alb_arn, ssl_enabled):
    alb_desc = elbv2_client.describe_listeners(
        LoadBalancerArn=alb_arn
    )
    return __get_listener(alb_desc, ssl_enabled)


def get_production_color(listener):
    """
    Récupère la couleur actuellement en production
    :param listener:    listener actuel
    :type listener:     dict
    :return:            BLUE/GREEN
    :rtype:             str
    """
    current_target_group_arn = __get_default_forward_target_group_arn_from_listener(listener)
    return __get_color_from_resource(current_target_group_arn)

def get_production_type(listener):
    """
    Récupère le type actuellement en production
    :param listener:    listener actuel
    :type listener:     dict
    :return:            default/maintenance
    :rtype:             str
    """
    current_target_group_arn = __get_default_forward_target_group_arn_from_listener(listener)
    return get_type_from_resource(current_target_group_arn)


def __get_listener(listeners, ssl_enabled):
    """
    Récupère le listener qui contient les règles de redirection vers les services
    :param listeners:   liste de listener disponible
    :type listeners:    dict
    :param ssl_enabled: indique si le ssl est activé ou non
    :type ssl_enabled:  bool
    :return:            listener correspondant au port et protocol donné
    :rtype:             dict
    """
    protocol_port = constant.HTTPS_TUPLE if ssl_enabled else constant.HTTP_TUPLE
    # Careful if we got multiple http listener
    for listener in listeners['Listeners']:
        if (listener['Protocol'], listener['Port']) == protocol_port:
            return listener


def __get_default_forward_target_group_arn_from_listener(listener):
    # Careful if we got multiple forward target group
    for action in listener['DefaultActions']:
        if action['Type'] == 'forward':
            return action['TargetGroupArn']


def __get_tags_from_resource(resource_arn):
    """
    Récupère les tags d'une ressource donnée
    :param resource_arn:    Ressource AWS arn
    :type resource_arn:     str
    :return:                tag value
    :rtype:                 str
    """
    tag_desc = elbv2_client.describe_tags(
        ResourceArns=[resource_arn]
    )
    tags = tag_desc['TagDescriptions'][0]['Tags']
    return tags

def __get_tag_value_from_resource(resource_arn, tag_name):
    tags = __get_tags_from_resource(resource_arn)
    for tag in tags:
        if tag['Key'] == tag_name:
            return tag['Value']


def __get_color_from_resource(resource_arn):
    """
    Récupère la couleur d'une ressource donnée
    :param resource_arn:    Ressource AWS arn
    :type resource_arn:     str
    :return:                BLUE/GREEN
    :rtype:                 str
    """
    return __get_tag_value_from_resource(resource_arn, constant.TARGET_GROUP_COLOR_TAG_NAME)


def get_type_from_resource(resource_arn):
    """
    Récupère le type d'une ressource donnée
    :param resource_arn:    Ressource AWS arn
    :type resource_arn:     str
    :return:                default/maintenance
    :rtype:                 str
    """
    return __get_tag_value_from_resource(resource_arn, constant.TARGET_GROUP_TYPE_TAG_NAME)


# ~~~~~~~~~~~~~~~~ Rules ~~~~~~~~~~~~~~~~


# Récupère les règles qui n'ont pas une couleur dans l'url
# ex : blue.beta.verde -> NON ; beta.verde -> OUI
def get_uncolored_rules(listener):
    rules_desc = elbv2_client.describe_rules(
        ListenerArn=listener['ListenerArn']
    )
    uncolored_rules = []
    for rule in rules_desc['Rules']:
        is_colored = False
        for condition in rule['Conditions']:
            host = condition.get('HostHeaderConfig', None)
            if host:
                is_colored = any(__is_colored_host_header_value(v) for v in host['Values'])

        if not is_colored:
          uncolored_rules.append(rule)
    return uncolored_rules


def __is_colored_host_header_value(value):
    return constant.BLUE in value or constant.GREEN in value


# ~~~~~~~~~~~~~~~~ TARGET GROUP ~~~~~~~~~~~~~~~~

def get_target_group_with_type_color_and_workspace(tg_type, color, workspace):
    """
    Récupère un target group ayant un type et une couleur précise
    :param tg_type: Type recherché
    :type tg_type:  str
    :param color:   Couleur recherché
    :type color:    str
    :param workspace: Workspace
    :type workspace:  str
    :return:        Target group trouvé
    :rtype:         dict
    """

    tag_filter = \
        [
            {
                'Key': 'Type',
                'Values': [
                    tg_type.lower(),
                ]
            },
            {
                'Key': 'Workspace',
                'Values': [
                    workspace.lower(),
                ]
            }
        ]

    if color:
        tag_filter.append({
            'Key': 'Color',
            'Values': [
                color.lower(),
            ]
        })

    response = tagging_client.get_resources(
        TagFilters=tag_filter,
        ResourceTypeFilters=[
            'elasticloadbalancing:targetgroup',
        ],
    )

    if len(response['ResourceTagMappingList']) != 1:
        raise Exception('Expected one target group with type {}, color {}, and workspace {}. But found {}'
                        .format(tg_type, color, workspace, str(len(response['ResourceTagMappingList']))))

    return response['ResourceTagMappingList'][0]['ResourceARN']



import boto3
from . import constant as constant

ses_client = boto3.client('ses', region_name='eu-central-1')


def send_mail_to_developers(message_content):
    ses_client.send_email(
        Source=constant.FROM_MAIL,
        Destination=__build_destination(),
        Message=__build_message_from_content(message_content)
    )


def __build_message_from_content(content):
    return {
        'Body': {
            'Text': {
                'Charset': constant.DEFAULT_CHARSET,
                'Data': content,
            },
        },
        'Subject': {
            'Charset': constant.DEFAULT_CHARSET,
            'Data': '⚠️ ERROR ON DEPLOYMENT SCRIPT ⚠️',
        },
    }


def __build_destination():
    return {
        'ToAddresses': [
            constant.DEVELOPERS_MAIL,
        ],
    }



import boto3

from . import constant as constant
from .deployment_manager \
    import EcsService

ecs_client = boto3.client('ecs')
application_autoscaling_client = boto3.client('application-autoscaling')


def get_services_from_cluster(cluster_name, max_results=100):
    return ecs_client.list_services(
        cluster=cluster_name,
        maxResults=max_results
    )


def get_services_arn_from_query(q, cluster_name):
    founded_services = []
    services = get_services_from_cluster(cluster_name)
    for service_arn in services['serviceArns']:
        if q.upper() in service_arn.upper():
            founded_services.append(service_arn)
    return founded_services


# Récupère les arn de tous les services ecs d'un cluster pour une couleur donnée
def get_services_arn_for_color(color, cluster_name):
    colored_services = []
    services = get_services_from_cluster(cluster_name)
    for service_arn in services['serviceArns']:
        if color.upper() in service_arn.upper():
            colored_services.append(service_arn)
    return colored_services


def get_service_max_capacity_from_service_arn(service_arn):
    tag_description_result = ecs_client.list_tags_for_resource(resourceArn=service_arn)
    tags = tag_description_result.get('tags')
    max_capacity_value = None
    for i in range(len(tags)):
        if tags[i].get("key") == "MaxCapacity":
            max_capacity_value = int(tags[i].get("value"))
            break

    return max_capacity_value or constant.DEFAULT_MAX_CAPACITY


def get_service_resource_id_from_service_arn(service_arn):
    return str(service_arn).split(':')[5]


# récupère le nom du repository de l'image des services
def get_map_of_repo_name_service(color, cluster_name):
    repo_name_service_map = {}
    services = get_services_from_cluster(cluster_name)
    for service_arn in services['serviceArns']:
        if color.upper() in service_arn.upper():
            service = ecs_client.describe_services(
                cluster=cluster_name,
                services=[service_arn]
            )
            taskDefinition = ecs_client.describe_task_definition(
                taskDefinition=service['services'][0]['taskDefinition']
            )
            containerDefinitions = taskDefinition['taskDefinition']['containerDefinitions']

            if containerDefinitions:
                image = containerDefinitions[0]['image']

                # Extrait le nom du repository de l'image
                # (ex: 721041490777.dkr.ecr.us-east-1.amazonaws.com/lcdp-api-gateway:BLUE)
                repository_name = image.split('/')[-1].split(':')[0]

                ecsService = EcsService(ecs_client=ecs_client,
                                        application_autoscaling_client=application_autoscaling_client,
                                        cluster_name=cluster_name,
                                        service_arn=service_arn,
                                        max_capacity=get_service_max_capacity_from_service_arn(service_arn),
                                        resource_id=get_service_resource_id_from_service_arn(service_arn))

                repo_name_service_map[repository_name] = ecsService

    return repo_name_service_map



from .constant import *


# Récupère les tags pour une ressource donnée
def get_tags_from_resource(resource, client):
    tag_descriptions_result = client.describe_tags(ResourceArns=[resource])
    for tag_desc in tag_descriptions_result['TagDescriptions']:
        return tag_desc['Tags']


# Récupère les attribues type et color dans une liste de tag
def get_type_and_color_for_resource(arn, client):
    tags = get_tags_from_resource(arn, client)
    return get_type_tag(tags), get_color_tag(tags)


# Récupère l'attribue type dans une liste de tag
def get_type_tag(tags):
    for tag in tags:
        if tag['Key'].upper() == TARGET_GROUP_TYPE_TAG_NAME.upper():
            return tag['Value']

# Récupère l'attribue color dans une liste de tag
def get_color_tag(tags):
    for tag in tags:
        if tag['Key'].upper() == TARGET_GROUP_COLOR_TAG_NAME.upper():
            return tag['Value']



from datetime import datetime, timezone, timedelta

import boto3

import logging

cloudwatch_client = boto3.client('cloudwatch')

def __search_expression(env, env_color, metric_name, aggregator):
    return "SEARCH('{{LCDP-SMUGGLER,ServiceEnvironment,ServiceVersion,SmugglerId}} MetricName=\"{metric_name}\" ServiceEnvironment=\"{service_environment}\" ServiceVersion=\"{service_version}\"', '{aggregator}', 30)".format(metric_name=metric_name, service_environment=env, service_version=env_color, aggregator=aggregator)

def __get_smugglers_metric_value(response, metric_name, aggregator):
    values = [x['Values'][0] for x in response['MetricDataResults'] if x['Id'] == metric_name and len(x['Values']) > 0]
    if len(values) > 0:
        return aggregator(values)
    return None

def get_smuggler_metrics(env, env_color):
    end_time = datetime.now(timezone.utc)
    start_time = end_time - timedelta(minutes=3)

    response = cloudwatch_client.get_metric_data(
        MetricDataQueries=[
            {
                'Id': 'active_jobs',
                "Expression": __search_expression(env, env_color, "ActiveJobs", 'Maximum'),
            },
            {
                'Id': 'pending_jobs',
                "Expression": __search_expression(env, env_color, "PendingJobs", 'Maximum'),
            },
        ],
        StartTime=start_time,
        EndTime=end_time,
        ScanBy='TimestampDescending'
    )

    metrics = dict()

    try:
        active_jobs = __get_smugglers_metric_value(response, 'active_jobs', max)
        if active_jobs is not None:
            metrics['active_jobs'] = active_jobs
    except (Exception):
        logging.exception("An error occured while retrieving 'active_jobs'")

    try:
        pending_jobs = __get_smugglers_metric_value(response, 'pending_jobs', max)
        if pending_jobs is not None:
            metrics['pending_jobs'] = pending_jobs
    except (Exception):
        logging.exception("An error occured while retrieving 'pending_jobs'")

    return metrics



import boto3
from . import constant as constant

ecr_client = boto3.client('ecr')


# Récupère le nom des ECR qui sont des services
# Un service commence par 'lcdp-'
def get_service_repositories_name():
    repositories = ecr_client.describe_repositories()
    service_repositories = []
    for repository in repositories['repositories']:
        if repository['repositoryName'].startswith(constant.ECR_SERVICE_PREFIX):
            service_repositories.append(repository['repositoryName'])
    return service_repositories


# Récupère une image possédant un tag précis
def get_repository_image_for_tag(repository_name, tag):
    images = ecr_client.list_images(
        repositoryName=repository_name,
        # TODO: Review if one day we got more than 1000 ecr images !
        maxResults=1000
    )
    for image in images['imageIds']:
        if 'imageTag' in image and image['imageTag'].upper() == tag.upper():
            return image


# Récupère la liste des images pour lesquelles le tag n'est pas le même que la couleur active
def find_mismatched_repositories_between_tag_and_color(repositories_name, tag, color):
    mismatched_repositories_name = []

    for repository_name in repositories_name:
        tag_image = get_repository_image_for_tag(repository_name, tag)
        color_image = get_repository_image_for_tag(repository_name, color)

        if tag_image and color_image and tag_image['imageDigest'] != color_image['imageDigest']:
            print('Mismatched image for repository: {}'.format(repository_name))
            mismatched_repositories_name.append(repository_name)

    return mismatched_repositories_name


# Récupère le manifest d'une image
def get_image_manifest(repository_name, image):
    detailed_image = ecr_client.batch_get_image(
        repositoryName=repository_name,
        imageIds=[image]
    )
    return detailed_image['images'][0]['imageManifest']


from setuptools import setup

setup(
    name='lcdp_deployment_manager',  # How you named your package folder
    packages=['lcdp_deployment_manager'],  # Chose the same as "name"
    version='1.2.8',  # Start with a small number and increase it with every change you make
    license='MIT',  # Chose a license from here: https://help.github.com/articles/licensing-a-repository
    # description: Give a short description about your library
    long_description_content_type="text/plain",
    description='High level utilities to get/set AWS infrastructure items on prod',
    long_description='High level utilities to get/set AWS infrastructure items on prod',
    author='Géry THRASIBULE',  # Author's name
    author_email='g.thrasibule@lecomptoirdespharmacies.fr',  # Author's E-Mail
    # url: Provide either the link to your github or to your website
    url='https://github.com/LeComptoirDesPharmacies/lcdp-deployment-manager',
    # download_url: Up the version (use the same version as in "version")
    download_url='https://github.com/LeComptoirDesPharmacies/lcdp-deployment-manager/archive/1.2.7.tar.gz',
    keywords=['AWS', 'Python', 'Deployment'],  # Keywords that define your package best
    classifiers=[
        'Development Status :: 3 - Alpha',
        # Chose either "3 - Alpha", "4 - Beta" or "5 - Production/Stable" as the current state of your package
        'Intended Audience :: Developers',  # Define that your audience are developers
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: MIT License',  # Pick a license
        'Programming Language :: Python :: 3',  # Specify which python versions that you want to support
    ],
    install_requires=['boto3'],  # list your package's dependencies
)



import time

from . import manage_ecs as ecs_manager


# Démarre tous les services d'un environement et attend qu'il soit entièrement up
def start_environment_and_wait_for_health(environment):
    environment.start_up_services()
    environment.wait_for_services_health()


# Passe d'un environnement à l'autre en modifiant les targets groups des règles du listener
def do_balancing(deployment_manager, from_environment, to_environment):
    print("Do balancing from environment {} to environment {}".format(from_environment.color, to_environment.color))
    deployment_manager.update_rule_target_group(
        expected_rule_type=from_environment.target_group_type,
        expected_rule_color=from_environment.color,
        new_target_group_arn=to_environment.target_group_arn
    )


def deploy_services_of_repositories_name(environment, repositories_name):
    print("Deploy services for repositories {}".format(repositories_name))

    services_to_start = []

    repo_name_service_map = ecs_manager.get_map_of_repo_name_service(environment.color, environment.cluster_name)

    # récupère les services à démarrer issus des repositories donnés
    for repo_name in repositories_name:
        if repo_name in repo_name_service_map:
            service = repo_name_service_map[repo_name]
            services_to_start.append(service)

    if services_to_start:
        for service in services_to_start:
            print("Start service {}".format(service.resource_id))
            service.start()

        # Wait for all service receive startup
        time.sleep(10)
        environment.wait_for_services_health()



# Constant.py

# Color
BLUE = 'blue'
GREEN = 'green'

# LISTENER
HTTPS_TUPLE = ('HTTPS', 443)
HTTP_TUPLE = ('HTTP', 80)

# Target group
TARGET_GROUP_COLOR_TAG_NAME = 'Color'
TARGET_GROUP_TYPE_TAG_NAME = 'Type'
# Default type means 'api gateway'
TARGET_GROUP_DEFAULT_TYPE = 'default'
TARGET_GROUP_MAINTENANCE_TYPE = 'maintenance'

# ECR
ECR_SERVICE_PREFIX = 'lcdp-'

# ECS
DEFAULT_DESIRED_COUNT = 2
DEFAULT_MAX_CAPACITY = 4
HEALTHCHECK_RETRY_LIMIT = 26
HEALTHCHECK_SLEEPING_TIME = 30
ECS_SERVICE_NAMESPACE = 'ecs'

# SES
FROM_MAIL = 'no-reply@lecomptoirdespharmacies.fr'
DEVELOPERS_MAIL = 'webmaster@lecomptoirdespharmacies.fr'
DEFAULT_CHARSET = 'UTF-8'

# ASG
DEFAULT_SCALABLE_DIMENSION = 'ecs:service:DesiredCount'
